@@ -53,6 +53,7 @@ pub enum Function {
     ToUpper,
     ToLower,
     IncMap,
+    Exit,
 }
 
 #[derive(Copy, Clone, Debug, PartialEq, Eq)]
@@ -231,7 +232,8 @@ static_map!(
     ["index", Function::SubstrIndex],
     ["toupper", Function::ToUpper],
     ["tolower", Function::ToLower],
-    ["system", Function::System]
+    ["system", Function::System],
+    ["exit", Function::Exit]
 );
 
 impl<'a> TryFrom<&'a str> for Function {
@@ -357,7 +359,7 @@ impl Function {
                     (Str, Str) => smallvec![Str; 2],
                     (Int, Int) | (Null, Int) | (Int, Null) | (Null, Null) => smallvec![Int; 2],
                     (_, Str) | (Str, _) | (Float, _) | (_, Float) => smallvec![Float; 2],
-                    _ => return err!("invalid input spec for comparison op: {:?}", &incoming[..]),
+                    _ => return err!("invalid input spec for comparison op: {:?}", incoming),
                 },
                 Int,
             ),
@@ -368,12 +370,12 @@ impl Function {
             Contains => match incoming[0] {
                 MapIntInt | MapIntStr | MapIntFloat => (smallvec![incoming[0], Int], Int),
                 MapStrInt | MapStrStr | MapStrFloat => (smallvec![incoming[0], Str], Int),
-                _ => return err!("invalid input spec fo Contains: {:?}", &incoming[..]),
+                _ => return err!("invalid input spec fo Contains: {:?}", incoming),
             },
             Delete => match incoming[0] {
                 MapIntInt | MapIntStr | MapIntFloat => (smallvec![incoming[0], Int], Int),
                 MapStrInt | MapStrStr | MapStrFloat => (smallvec![incoming[0], Str], Int),
-                _ => return err!("invalid input spec fo Delete: {:?}", &incoming[..]),
+                _ => return err!("invalid input spec fo Delete: {:?}", incoming),
             },
             IncMap => {
                 let map = incoming[0];
@@ -394,10 +396,7 @@ impl Function {
                 if incoming.len() == 1 && incoming[0].is_array() {
                     (smallvec![incoming[0]], Int)
                 } else {
-                    return err!(
-                        "invalid input spec for delete (of a map): {:?}",
-                        &incoming[..]
-                    );
+                    return err!("invalid input spec for delete (of a map): {:?}", incoming);
                 }
             }
             Srand => (smallvec![Int], Int),
@@ -429,12 +428,13 @@ impl Function {
             ToUpper | ToLower | EscapeCSV | EscapeTSV => (smallvec![Str], Str),
             Substr => (smallvec![Str, Int, Int], Str),
             Match => (smallvec![Str, Str], Int),
+            Exit => (smallvec![Int], Null),
             // Split's second input can be a map of either type
             Split => {
                 if let MapIntStr | MapStrStr = incoming[1] {
                     (smallvec![Str, incoming[1], Str], Int)
                 } else {
-                    return err!("invalid input spec for split: {:?}", &incoming[..]);
+                    return err!("invalid input spec for split: {:?}", incoming);
                 }
             }
             JoinCols => (smallvec![Int, Int, Str], Str),
@@ -450,7 +450,7 @@ impl Function {
             IntFunc(bw) => bw.arity(),
             UpdateUsedFields | Rand | ReseedRng | ReadErrStdin | NextlineStdin | NextFile
             | ReadLineStdinFused => 0,
-            ToUpper | ToLower | Clear | Srand | System | HexToInt | ToInt | EscapeCSV
+            Exit | ToUpper | ToLower | Clear | Srand | System | HexToInt | ToInt | EscapeCSV
             | EscapeTSV | Close | Length | ReadErr | ReadErrCmd | Nextline | NextlineCmd
             | Unop(_) => 1,
             SetFI | SubstrIndex | Match | Setcol | Binop(_) => 2,
@@ -497,7 +497,7 @@ impl Function {
                 Ok(Scalar(BaseTy::Str).abs())
             }
             IncMap => Ok(step_arith(&types::val_of(&args[0])?, &args[2])),
-            SetFI | UpdateUsedFields | NextFile | ReadLineStdinFused | Close => Ok(None),
+            Exit | SetFI | UpdateUsedFields | NextFile | ReadLineStdinFused | Close => Ok(None),
         }
     }
 }
@@ -586,7 +586,7 @@ impl<'a> Variables<'a> {
 
     pub fn store_int(&mut self, var: Variable, i: Int) -> Result<()> {
         use Variable::*;
-        Ok(match var {
+        match var {
             ARGC => self.argc = i,
             NF => self.nf = i,
             NR => self.nr = i,
@@ -595,7 +595,8 @@ impl<'a> Variables<'a> {
             RLENGTH => self.rlength = i,
             PID => self.pid = i,
             FI | ORS | OFS | FS | RS | FILENAME | ARGV => return err!("var {} not an int", var),
-        })
+        }
+        Ok(())
     }
 
     pub fn load_str(&self, var: Variable) -> Result<Str<'a>> {
@@ -614,7 +615,7 @@ impl<'a> Variables<'a> {
 
     pub fn store_str(&mut self, var: Variable, s: Str<'a>) -> Result<()> {
         use Variable::*;
-        Ok(match var {
+        match var {
             FS => self.fs = s,
             OFS => self.ofs = s,
             ORS => self.ors = s,
@@ -623,7 +624,8 @@ impl<'a> Variables<'a> {
             FI | PID | ARGC | ARGV | NF | NR | FNR | RSTART | RLENGTH => {
                 return err!("var {} not a string", var)
             }
-        })
+        };
+        Ok(())
     }
 
     pub fn load_intmap(&self, var: Variable) -> Result<IntMap<Str<'a>>> {
@@ -639,7 +641,10 @@ impl<'a> Variables<'a> {
     pub fn store_intmap(&mut self, var: Variable, m: IntMap<Str<'a>>) -> Result<()> {
         use Variable::*;
         match var {
-            ARGV => Ok(self.argv = m),
+            ARGV => {
+                self.argv = m;
+                Ok(())
+            }
             FI | PID | ORS | OFS | ARGC | NF | NR | FNR | FS | RS | FILENAME | RSTART | RLENGTH => {
                 err!("var {} is not an int-keyed map", var)
             }
@@ -659,7 +664,10 @@ impl<'a> Variables<'a> {
     pub fn store_strmap(&mut self, var: Variable, m: StrMap<'a, Int>) -> Result<()> {
         use Variable::*;
         match var {
-            FI => Ok(self.fi = m),
+            FI => {
+                self.fi = m;
+                Ok(())
+            }
             ARGV | PID | ORS | OFS | ARGC | NF | NR | FNR | FS | RS | FILENAME | RSTART
             | RLENGTH => {
                 err!("var {} is not a string-keyed map", var)
