@@ -277,12 +277,7 @@ impl Generator {
             .shared
             .module
             .declare_function(name, Linkage::Export, &sig)
-            .map_err(|e| {
-                CompileError(format!(
-                    "failed to declare main function: {}",
-                    e.to_string()
-                ))
-            })?;
+            .map_err(|e| CompileError(format!("failed to declare main function: {}", e)))?;
         let prelude = Prelude {
             sig,
             refs: smallvec![PLACEHOLDER],
@@ -378,7 +373,7 @@ impl Generator {
     }
 
     /// Initialize a new user-defined function and prepare it for full code generation.
-    fn create_view<'a>(&'a mut self, Prelude { sig, refs, n_args }: Prelude) -> View<'a> {
+    fn create_view(&mut self, Prelude { sig, refs, n_args }: Prelude) -> View {
         // Initialize a frame for the function at the given offset, declare variables corresponding
         // to globals and params, return a View to proceed with the rest of code generation.
         let n_params = sig.params.len();
@@ -470,7 +465,7 @@ impl Generator {
                 .shared
                 .module
                 .declare_function(name.as_str(), Linkage::Local, &sig)
-                .map_err(|e| CompileError(format!("cranelift module error: {}", e.to_string())))?;
+                .map_err(|e| CompileError(format!("cranelift module error: {}", e)))?;
 
             self.funcs.push(Some(Prelude {
                 sig,
@@ -529,7 +524,7 @@ impl<'a> View<'a> {
                 while let Some(e) = walker.next_edge(&insts.cfg) {
                     let (_, next) = insts.cfg.edge_endpoints(e).unwrap();
                     let bb = bbs[next.index()];
-                    if let Some(e) = insts.cfg.edge_weight(e).unwrap().clone() {
+                    if let Some(e) = *insts.cfg.edge_weight(e).unwrap() {
                         tcase = Some(((e, compile::Ty::Int), bb));
                     } else {
                         ecase = Some(bb);
@@ -578,7 +573,7 @@ impl<'a> View<'a> {
         self.builder.seal_all_blocks();
         self.builder.finalize();
         if DUMP_IR {
-            eprintln!("{}", self.builder.display(None));
+            eprintln!("{}", self.builder.func);
         }
         Ok(())
     }
@@ -731,14 +726,12 @@ impl<'a> View<'a> {
             }
             Ret(reg, ty) => {
                 let mut v = self.get_val((*reg, *ty))?;
-                if matches!(
-                    self.f.vars.get_mut(&(*reg, *ty)).map(|x| {
-                        // if this is a local, we want to avoid dropping it.
-                        x.kind.skip_drop();
-                        &x.kind
-                    }),
-                    Some(VarKind::Param) | Some(VarKind::Global)
-                ) {
+                let test = self.f.vars.get_mut(&(*reg, *ty)).map(|x| {
+                    // if this is a local, we want to avoid dropping it.
+                    x.kind.skip_drop();
+                    &x.kind
+                });
+                if let Some(VarKind::Param) | Some(VarKind::Global) = test {
                     // This was passed in as a parameter, so us returning it will introduce a new
                     // reference.
                     self.ref_val(*ty, v);
@@ -806,7 +799,7 @@ impl<'a> View<'a> {
     }
 
     fn execute_actions(&mut self) -> Result<()> {
-        let header_actions = mem::replace(&mut self.f.header_actions, Default::default());
+        let header_actions = mem::take(&mut self.f.header_actions);
         for EntryDeclaration { var, ty } in header_actions {
             use compile::Ty::*;
             let cl_ty = self.get_ty(ty);
@@ -1220,21 +1213,11 @@ struct RegistrationState {
 
 impl Backend for RegistrationState {
     type Ty = ();
-    fn void_ptr_ty(&self) -> () {
-        ()
-    }
-    fn ptr_to(&self, (): ()) -> () {
-        ()
-    }
-    fn usize_ty(&self) -> () {
-        ()
-    }
-    fn u32_ty(&self) -> () {
-        ()
-    }
-    fn get_ty(&self, _ty: compile::Ty) -> () {
-        ()
-    }
+    fn void_ptr_ty(&self) {}
+    fn ptr_to(&self, (): ()) {}
+    fn usize_ty(&self) {}
+    fn u32_ty(&self) {}
+    fn get_ty(&self, _ty: compile::Ty) {}
 
     fn register_external_fn(
         &mut self,
@@ -1364,7 +1347,8 @@ impl<'a> CodeGenerator for View<'a> {
     }
 
     fn call_void(&mut self, func: *const u8, args: &mut [Self::Val]) -> Result<()> {
-        Ok(self.call_external_void(func, args))
+        self.call_external_void(func, args);
+        Ok(())
     }
 
     // TODO We may eventually want to remove the `Result` return value here
@@ -1399,7 +1383,7 @@ impl<'a> CodeGenerator for View<'a> {
         args: &[Ref],
     ) -> Result<()> {
         // For empty args, just delegate to print_all
-        if args.len() == 0 {
+        if args.is_empty() {
             return self.print_all(output, &[*fmt]);
         }
         let (arg_slot, type_slot, num_args) = self.bundle_printf_args(args)?;
@@ -1428,7 +1412,7 @@ impl<'a> CodeGenerator for View<'a> {
 
     fn sprintf(&mut self, dst: &StrReg, fmt: &StrReg, args: &[Ref]) -> Result<()> {
         // For empty args, just move fmt into dst.
-        if args.len() == 0 {
+        if args.is_empty() {
             return self.mov(compile::Ty::Str, dst.reflect().0, fmt.reflect().0);
         }
 
