@@ -16,7 +16,7 @@ use crate::runtime::{
 };
 use crate::{
     builtins::Variable,
-    common::{FileSpec, Result},
+    common::{CancelSignal, Cleanup, FileSpec, Notification, Result},
     compile::Ty,
     pushdown::FieldSet,
 };
@@ -128,6 +128,7 @@ pub(crate) fn register_all(cg: &mut impl Backend) -> Result<()> {
         seed_rng(rt_ty, int_ty) -> int_ty;
         reseed_rng(rt_ty) -> int_ty;
 
+        exit(rt_ty, int_ty);
         run_system(str_ref_ty) -> int_ty;
         print_all_stdout(rt_ty, pa_args_ty, int_ty);
         print_all_file(rt_ty, pa_args_ty, int_ty, str_ref_ty, int_ty);
@@ -284,7 +285,7 @@ macro_rules! fail {
         #[cfg(not(test))]
         {
             eprintln_ignore!("failure in runtime {}. Halting execution", format!($($es),*));
-            exit!($rt, 1, format!($($es),*))
+            exit!($rt, 1)
         }
     }}
 }
@@ -314,17 +315,31 @@ macro_rules! try_silent_abort {
 
 macro_rules! exit {
     ($runtime:expr) => {
-        exit!($runtime, 0, "")
+        exit!($runtime, 0)
     };
-    ($runtime:expr, $code:expr, $msg:expr) => {{
-        let rt = $runtime as *const _ as *mut Runtime;
-        let concurrent = (*rt).concurrent;
-        if concurrent {
-            // Use panic to allow 'graceful' shutdown of other worker threads.
-            panic!("")
+    ($runtime:expr, $code:expr) => {{
+        // XXX: revisit if this is undefined behavior having &mut Runtime in scope even if it is
+        // not used after the call to drop_in_place.
+        let rt_raw = $runtime as *mut Runtime;
+        let rt = &mut *rt_raw;
+        let code = $code;
+        if rt.concurrent {
+            let pid = rt.core.vars.pid;
+            rt.cancel_signal.cancel(code);
+            std::ptr::drop_in_place(rt_raw);
+            if pid == 1 {
+                // We are the main thread. Drop on `rt` should have waited for other threads to exit.
+                // All that's left is for us to abort.
+                std::process::exit(code)
+            } else {
+                // Block forever. Let the main thread exit.
+                let n = Notification::default();
+                n.wait();
+                unreachable!()
+            }
         } else {
-            std::ptr::drop_in_place(rt);
-            std::process::exit($code)
+            std::ptr::drop_in_place(rt_raw);
+            std::process::exit(code)
         }
     }};
 }
@@ -354,6 +369,7 @@ pub(crate) trait IntoRuntime {
         ff: impl runtime::writers::FileFactory,
         used_fields: &FieldSet,
         named_columns: Option<Vec<&[u8]>>,
+        cancel_signal: CancelSignal,
     ) -> Runtime<'a>;
 }
 
@@ -365,6 +381,7 @@ macro_rules! impl_into_runtime {
                 ff: impl runtime::writers::FileFactory,
                 used_fields: &FieldSet,
                 named_columns: Option<Vec<&[u8]>>,
+                cancel_signal: CancelSignal,
             ) -> Runtime<'a> {
                 Runtime {
                     concurrent: false,
@@ -373,6 +390,8 @@ macro_rules! impl_into_runtime {
                         FileRead::new(self, used_fields.clone(), named_columns),
                     )),
                     core: crate::interp::Core::new(ff),
+                    cleanup: Cleanup::null(),
+                    cancel_signal,
                 }
             }
         }
@@ -398,6 +417,8 @@ pub(crate) struct Runtime<'a> {
     pub(crate) input_data: InputData,
     #[allow(unused)]
     pub(crate) concurrent: bool,
+    pub(crate) cancel_signal: CancelSignal,
+    pub(crate) cleanup: Cleanup<Self>,
 }
 
 impl<'a> Runtime<'a> {
@@ -409,6 +430,16 @@ impl<'a> Runtime<'a> {
     }
 }
 
+impl<'a> Drop for Runtime<'a> {
+    fn drop(&mut self) {
+        mem::replace(&mut self.cleanup, Cleanup::null()).invoke(self);
+    }
+}
+
+pub(crate) unsafe extern "C" fn exit(runtime: *mut c_void, code: Int) {
+    exit!(runtime, code as i32);
+}
+
 pub(crate) unsafe extern "C" fn run_system(cmd: *mut U128) -> Int {
     let s: &Str = &*(cmd as *mut Str);
     s.with_bytes(runtime::run_command)
@@ -548,7 +579,7 @@ pub(crate) unsafe extern "C" fn split_str(
     if let Err(e) = runtime
         .core
         .regexes
-        .split_regex_strmap(&pat, &to_split, &into_arr)
+        .split_regex_strmap(pat, to_split, &into_arr)
     {
         fail!(runtime, "failed to split string: {}", e);
     }
@@ -679,9 +710,7 @@ pub(crate) unsafe extern "C" fn set_col(runtime: *mut c_void, col: Int, s: *mut
 
 pub(crate) unsafe extern "C" fn str_len(s: *mut c_void) -> usize {
     let s = &*(s as *mut Str);
-    let res = s.len();
-    mem::forget(s);
-    res
+    s.len()
 }
 
 pub(crate) unsafe extern "C" fn starts_with_const(
@@ -716,7 +745,6 @@ pub(crate) unsafe extern "C" fn match_pat(
         (*runtime).core.is_match_regex(s, pat),
         "match_pat:"
     );
-    mem::forget((s, pat));
     res as Int
 }
 
@@ -739,7 +767,6 @@ pub(crate) unsafe extern "C" fn match_pat_loc(
         (*runtime).core.match_regex(s, pat),
         "match_pat_loc:"
     );
-    mem::forget((s, pat));
     res as Int
 }
 
@@ -819,8 +846,12 @@ pub(crate) unsafe extern "C" fn substr(base: *mut U128, l: Int, r: Int) -> U128
     let base = &*(base as *mut Str);
     let len = base.len();
     let l = max(0, l - 1);
-    let r = min(len as Int, l.saturating_add(r)) as usize;
-    mem::transmute::<Str, U128>(base.slice(l as usize, r))
+    if l as usize >= len {
+        mem::transmute::<Str, U128>(Str::default())
+    } else {
+        let r = min(len as Int, l.saturating_add(r)) as usize;
+        mem::transmute::<Str, U128>(base.slice(l as usize, r))
+    }
 }
 
 pub(crate) unsafe extern "C" fn ref_str(s: *mut c_void) {
@@ -863,24 +894,21 @@ pub(crate) unsafe extern "C" fn float_to_str(f: Float) -> U128 {
 
 pub(crate) unsafe extern "C" fn str_to_int(s: *mut c_void) -> Int {
     let s = &*(s as *mut Str);
-    let res = runtime::convert::<&Str, Int>(&s);
-    res
+    runtime::convert::<&Str, Int>(s)
 }
 
 pub(crate) unsafe extern "C" fn hex_str_to_int(s: *mut c_void) -> Int {
     let s = &*(s as *mut Str);
-    let res = s.with_bytes(runtime::hextoi);
-    res
+    s.with_bytes(runtime::hextoi)
 }
 
 pub(crate) unsafe extern "C" fn str_to_float(s: *mut c_void) -> Float {
     let s = &*(s as *mut Str);
-    let res = runtime::convert::<&Str, Float>(&s);
-    res
+    runtime::convert::<&Str, Float>(s)
 }
 
 pub(crate) unsafe extern "C" fn load_var_str(rt: *mut c_void, var: usize) -> U128 {
-    let runtime = &*(rt as *mut Runtime);
+    let runtime = &mut *(rt as *mut Runtime);
     if let Ok(var) = Variable::try_from(var) {
         let res = try_abort!(runtime, runtime.core.vars.load_str(var));
         mem::transmute::<Str, U128>(res)
@@ -926,7 +954,7 @@ pub(crate) unsafe extern "C" fn store_var_int(rt: *mut c_void, var: usize, i: In
 }
 
 pub(crate) unsafe extern "C" fn load_var_intmap(rt: *mut c_void, var: usize) -> *mut c_void {
-    let runtime = &*(rt as *mut Runtime);
+    let runtime = &mut *(rt as *mut Runtime);
     if let Ok(var) = Variable::try_from(var) {
         let res = try_abort!(runtime, runtime.core.vars.load_intmap(var));
         mem::transmute::<IntMap<_>, *mut c_void>(res)
@@ -947,7 +975,7 @@ pub(crate) unsafe extern "C" fn store_var_intmap(rt: *mut c_void, var: usize, ma
 }
 
 pub(crate) unsafe extern "C" fn load_var_strmap(rt: *mut c_void, var: usize) -> *mut c_void {
-    let runtime = &*(rt as *mut Runtime);
+    let runtime = &mut *(rt as *mut Runtime);
     if let Ok(var) = Variable::try_from(var) {
         let res = try_abort!(runtime, runtime.core.vars.load_strmap(var));
         mem::transmute::<StrMap<_>, *mut c_void>(res)
@@ -969,12 +997,10 @@ pub(crate) unsafe extern "C" fn store_var_strmap(rt: *mut c_void, var: usize, ma
 
 macro_rules! str_compare_inner {
     ($name:ident, $op:tt) => {
-
         pub(crate) unsafe extern "C" fn $name(s1: *mut c_void, s2: *mut c_void) -> Int {
             let s1 = &*(s1 as *mut Str);
             let s2 = &*(s2 as *mut Str);
             let res = s1.with_bytes(|bs1| s2.with_bytes(|bs2| bs1 $op bs2)) as Int;
-            mem::forget((s1, s2));
             res
         }
     }
@@ -1017,7 +1043,7 @@ unsafe fn wrap_args<'a>(
         };
         let typed_arg: FormatArg = match ty {
             Ty::Int => mem::transmute::<usize, Int>(arg).into(),
-            Ty::Float => mem::transmute::<usize, Float>(arg).into(),
+            Ty::Float => Float::from_bits(arg as u64).into(),
             Ty::Str => mem::transmute::<usize, &Str>(arg).clone().into(),
             _ => fail!(
                 _rt,
