@@ -23,6 +23,7 @@ mod input_taint;
 pub mod interp;
 pub mod lexer;
 #[allow(unused_parens)] // Warnings appear in generated code
+#[allow(clippy::all)]
 pub mod parsing;
 pub mod pushdown;
 pub mod runtime;
@@ -36,7 +37,7 @@ use clap::{App, Arg};
 use arena::Arena;
 use cfg::Escaper;
 use codegen::intrinsics::IntoRuntime;
-use common::{ExecutionStrategy, Stage};
+use common::{CancelSignal, ExecutionStrategy, Stage};
 use runtime::{
     splitter::{
         batch::{ByteReader, CSVReader, InputFormat},
@@ -51,7 +52,7 @@ use std::mem;
 
 #[cfg(feature = "use_jemalloc")]
 #[global_allocator]
-static ALLOC: jemallocator::Jemalloc = jemallocator::Jemalloc;
+static ALLOC: tikv_jemallocator::Jemalloc = tikv_jemallocator::Jemalloc;
 
 macro_rules! fail {
     ($($t:tt)*) => {{
@@ -216,13 +217,18 @@ fn run_interp_with_context<'a>(
     ff: impl runtime::writers::FileFactory,
     num_workers: usize,
 ) {
-    let mut interp = match compile::bytecode(&mut ctx, stdin, ff, num_workers) {
-        Ok(ctx) => ctx,
-        Err(e) => fail!("bytecode compilation failure: {}", e),
+    let rc = {
+        let mut interp = match compile::bytecode(&mut ctx, stdin, ff, num_workers) {
+            Ok(ctx) => ctx,
+            Err(e) => fail!("bytecode compilation failure: {}", e),
+        };
+        match interp.run() {
+            Err(e) => fail!("fatal error during execution: {}", e),
+            Ok(0) => return,
+            Ok(n) => n,
+        }
     };
-    if let Err(e) = interp.run() {
-        fail!("fatal error during execution: {}", e);
-    }
+    std::process::exit(rc);
 }
 
 fn run_cranelift_with_context<'a>(
@@ -230,8 +236,9 @@ fn run_cranelift_with_context<'a>(
     stdin: impl IntoRuntime,
     ff: impl runtime::writers::FileFactory,
     cfg: codegen::Config,
+    signal: CancelSignal,
 ) {
-    if let Err(e) = compile::run_cranelift(&mut ctx, stdin, ff, cfg) {
+    if let Err(e) = compile::run_cranelift(&mut ctx, stdin, ff, cfg, signal) {
         fail!("error compiling cranelift: {}", e)
     }
 }
@@ -243,8 +250,9 @@ cfg_if::cfg_if! {
             stdin: impl IntoRuntime,
             ff: impl runtime::writers::FileFactory,
             cfg: codegen::Config,
+            signal: CancelSignal,
         ) {
-            if let Err(e) = compile::run_llvm(&mut ctx, stdin, ff, cfg) {
+            if let Err(e) = compile::run_llvm(&mut ctx, stdin, ff, cfg, signal) {
                 fail!("error compiling llvm: {}", e)
             }
         }
@@ -276,6 +284,7 @@ fn dump_bytecode(prog: &str, raw: &RawPrelude) -> String {
             CHUNK_SIZE,
             /*check_utf8=*/ false,
             ExecutionStrategy::Serial,
+            Default::default(),
         )),
         runtime::writers::default_factory(),
         /*num_workers=*/ 1,
@@ -285,11 +294,11 @@ fn dump_bytecode(prog: &str, raw: &RawPrelude) -> String {
     };
     let mut v = Vec::<u8>::new();
     for (i, func) in interp.instrs().iter().enumerate() {
-        write!(&mut v, "function {} {{\n", i).unwrap();
+        writeln!(&mut v, "function {} {{", i).unwrap();
         for (j, inst) in func.iter().enumerate() {
-            write!(&mut v, "\t[{:2}] {:?}\n", j, inst).unwrap();
+            writeln!(&mut v, "\t[{:2}] {:?}", j, inst).unwrap();
         }
-        write!(&mut v, "}}\n").unwrap();
+        writeln!(&mut v, "}}\n").unwrap();
     }
     String::from_utf8(v).unwrap()
 }
@@ -297,80 +306,106 @@ fn dump_bytecode(prog: &str, raw: &RawPrelude) -> String {
 fn main() {
     #[allow(unused_mut)]
     let mut app = App::new("frawk")
-        .version("0.4.4")
+        .version("0.4.5")
         .author("Eli R.")
         .about("frawk is a pattern scanning and (semi-structured) text processing language")
         .arg(Arg::new("program-file")
              .long("program-file")
              .short('f')
              .takes_value(true)
-             .multiple_occurrences(true))
+             .multiple_occurrences(true)
+             .help("Read the program source from the file program-file, instead of from the command line. Multiple '-f' options may be used"))
         .arg(Arg::new("opt-level")
              .long("opt-level")
              .short('O')
-             .about("the optimization level for the program. Positive levels determine the optimization level for LLVM. Level -1 forces bytecode interpretation")
-             .possible_values(&["0", "1", "2", "3"]))
-        .arg("--out-file=[FILE] 'the output file used in place of standard input'")
-        .arg("--utf8 'validate all input as UTF-8, returning an error if it is invalid'")
-        .arg("--dump-cfg 'print untyped SSA form for input program'")
-        .arg("--dump-bytecode 'print bytecode for input program'")
+             .takes_value(true)
+             .allow_hyphen_values(true)
+             .help("The optimization level for the program. Positive levels determine the optimization level for LLVM. Level `-1` forces bytecode interpretation")
+             .possible_values(&["-1", "0", "1", "2", "3"]))
+        .arg(Arg::new("out-file")
+             .long("out-file")
+             .takes_value(true)
+             .value_name("FILE")
+             .help("Write to specified output file instead of standard output"))
+        .arg(Arg::new("utf8")
+             .long("utf8")
+             .takes_value(false)
+             .help("Validate all input as UTF-8, returning an error if it is invalid"))
+        .arg(Arg::new("dump-cfg")
+             .long("dump-cfg")
+             .takes_value(false)
+             .help("Print untyped SSA form for input program"))
+        .arg(Arg::new("dump-bytecode")
+             .long("dump-bytecode")
+             .takes_value(false)
+             .help("Print bytecode for input program"))
         .arg(Arg::new("parse-header")
              .long("parse-header")
              .short('H')
              .takes_value(false)
-             .about("consume the first line of input and populate the `FI` variable with column names mapping to column indexes"))
+             .help("Consume the first line of input and populate the `FI` variable with column names mapping to column indexes"))
         .arg(Arg::new("input-format")
              .long("input-format")
              .short('i')
-             .possible_values(&["csv", "tsv"])
-             .about("Input is split according to the rules of (csv|tsv). $0 contains the unescaped line. Assigning to columns does nothing."))
+             .value_name("csv|tsv")
+             .conflicts_with("field-separator")
+             .help("Input is split according to the rules of (csv|tsv). $0 contains the unescaped line. Assigning to columns does nothing")
+             .possible_values(&["csv", "tsv"]))
         .arg(Arg::new("var")
              .short('v')
-             .multiple_occurrences(true)
-             .multiple_values(true)
              .takes_value(true)
-             .min_values(1)
-             .about("Has the form <identifier>=<chars>. A variable with the name <identifier> is assigned to an escaped string literal containing <chars> before the body of the frawk program is run."))
-        .arg("-F, --field-separator=[SEPARATOR] 'Field separator for frawk program.'")
+             .multiple_occurrences(true)
+             .value_name("var=val")
+             .help("Assign the value <val> to the variable <var>, before execution of the frawk program begins. Multiple '-v' options may be used"))
+        .arg(Arg::new("field-separator")
+             .long("field-separator")
+             .short('F')
+             .takes_value(true)
+             .value_name("FS")
+             .conflicts_with("input-format")
+             .help("Field separator `FS` for frawk program"))
         .arg(Arg::new("backend")
              .long("backend")
-             .short('b')
-             .about("The backend used to run the frawk program, ranging from fastest to compile and slowest to execute, and slowest to compile and fastest to execute. Cranelift is the default")
+             .short('B')
+             .help("The backend used to run the frawk program, ranging from fastest to compile and slowest to execute, and slowest to compile and fastest to execute. Cranelift is the default")
              .possible_values(&["interp", "cranelift", "llvm"]))
         .arg(Arg::new("output-format")
              .long("output-format")
              .short('o')
-             .possible_values(&["csv", "tsv"])
-             .about("If set, records output via print are escaped according to the rules of the corresponding format"))
+             .value_name("csv|tsv")
+             .help("If set, records output via print are escaped according to the rules of the corresponding format")
+             .possible_values(&["csv", "tsv"]))
         .arg(Arg::new("program")
-             .about("The frawk program to execute")
-             .index(1))
+             .index(1)
+             .help("The frawk program to execute"))
         .arg(Arg::new("input-files")
-             .about("Input files to be read by frawk program")
              .index(2)
-             .multiple_values(true))
+             .multiple_values(true)
+             .help("Input files to be read by frawk program"))
         .arg(Arg::new("parallel-strategy")
-             .about("Attempt to execute the script in parallel. Strategy r[ecord] parallelizes within the current input file. Strategy f[ile] parallelizes between input files.")
              .short('p')
+             .help("Attempt to execute the script in parallel. Strategy r[ecord] parallelizes within the current input file. Strategy f[ile] parallelizes between input files")
              .possible_values(&["r", "record", "f", "file"]))
         .arg(Arg::new("chunk-size")
              .long("chunk-size")
-             .about("Buffer size when reading input. This is present primarily for debugging purposes; it's possible that tuning this will help performance, but it should not be necessary.")
-             .takes_value(true))
+             .takes_value(true)
+             .help("Buffer size when reading input. This is present primarily for debugging purposes; it's possible that tuning this will help performance, but it should not be necessary"))
         .arg(Arg::new("arbitrary-shell")
-             .about("")
              .short('A')
              .long("arbitrary-shell")
-             .about("By default, strings that are passed to the shell via pipes or the 'system' function are restricted from potentially containing user input. This flag bypasses that check, for the cases where such a use is known to be safe.")
-             .takes_value(false))
+             .takes_value(false)
+             .help("By default, strings that are passed to the shell via pipes or the 'system' function are restricted from potentially containing user input. This flag bypasses that check, for the cases where such a use is known to be safe"))
         .arg(Arg::new("jobs")
-                .about("Number or worker threads to launch when executing in parallel, requires '-p' flag to be set. When using record-level parallelism, this value is an upper bound on the number of worker threads that will be spawned; the number of active worker threads is chosen dynamically.")
-                .short('j')
-                .requires("parallel-strategy")
-                .takes_value(true));
+             .short('j')
+             .requires("parallel-strategy")
+             .takes_value(true)
+             .help("Number or worker threads to launch when executing in parallel, requires '-p' flag to be set. When using record-level parallelism, this value is an upper bound on the number of worker threads that will be spawned; the number of active worker threads is chosen dynamically"));
     cfg_if::cfg_if! {
         if #[cfg(feature = "llvm_backend")] {
-            app = app.arg("--dump-llvm 'print LLVM-IR for the input program'");
+            app = app.arg(Arg::new("dump-llvm")
+             .long("dump-llvm")
+             .takes_value(false)
+             .help("Print LLVM-IR for the input program"));
         }
     }
     let matches = app.get_matches();
@@ -520,12 +555,13 @@ fn main() {
         return;
     }
     let check_utf8 = matches.is_present("utf8");
+    let signal = CancelSignal::default();
 
     // This horrid macro is here because all of the different ways of reading input are different
     // types, making functions hard to write. Still, there must be something to be done to clean
     // this up here.
     macro_rules! with_inp {
-        ($analysis:expr, $inp:ident, $body:expr) => {
+        ($analysis:expr, $inp:ident, $body:expr) => {{
             if input_files.len() == 0 {
                 let _reader: Box<dyn io::Read + Send> = Box::new(io::stdin());
                 match (ifmt, $analysis) {
@@ -536,6 +572,7 @@ fn main() {
                             chunk_size,
                             check_utf8,
                             exec_strategy,
+                            signal.clone(),
                         );
                         $body
                     }
@@ -555,6 +592,7 @@ fn main() {
                                     chunk_size,
                                     check_utf8,
                                     exec_strategy,
+                                    signal.clone(),
                                 );
                                 $body
                             } else {
@@ -565,6 +603,7 @@ fn main() {
                                     chunk_size,
                                     check_utf8,
                                     exec_strategy,
+                                    signal.clone(),
                                 );
                                 $body
                             }
@@ -592,6 +631,7 @@ fn main() {
                     chunk_size,
                     check_utf8,
                     exec_strategy,
+                    signal.clone(),
                 );
                 $body
             } else {
@@ -614,6 +654,7 @@ fn main() {
                                     chunk_size,
                                     check_utf8,
                                     exec_strategy,
+                                    signal.clone(),
                                 );
                                 $body
                             } else {
@@ -624,6 +665,7 @@ fn main() {
                                     chunk_size,
                                     check_utf8,
                                     exec_strategy,
+                                    signal.clone(),
                                 );
                                 $body
                             }
@@ -648,7 +690,7 @@ fn main() {
                     }
                 }
             }
-        };
+        }};
     }
 
     let a = Arena::default();
@@ -682,6 +724,7 @@ fn main() {
                                 opt_level: opt_level as usize,
                                 num_workers,
                             },
+                            signal,
                     ));
                 } else {
                     fail!("backend specified as LLVM, but compiled without LLVM support");
@@ -700,6 +743,7 @@ fn main() {
                     opt_level: opt_level as usize,
                     num_workers,
                 },
+                signal,
             ));
         }
         Some(b) => {
